# CPU type
set cpu_type TMS320C674x

# Flash support
set flash_types { 0 2 }

# target-side code of flash loader
set FlashLoadSuppCode $mypath/trgcode/flashl138.out

# target is little endian
set TargetBigEndian 0

# PC register name
set pc_name PC

# base address of SPI0 in CPU's address space
set spi0_base 0x01C41000

# base address of SPI1 in CPU's address space
set spi1_base 0x01F0E000

# base address of I2C0 in CPU's address space
set i2c0_base 0x01C22000

# base address of I2C1 in CPU's address space
set i2c1_base 0x01E28000

# Word width (0 - 8 bit, 1 - 16, 2 - 32)
set word_width 0

# base address of EMIF control registers in CPU's address space
set emif_base 0x68000000


# LPSC Enable Function
proc PSC_LPSC_Enable { DEV PSC_NUM PD LPSC_num is_core } {

  if { $is_core != 0 } {
    set DT 0x103
    set MS 0x11F
  } else {
    set DT 0x03
    set MS 0x1F
  }

  if { $PSC_NUM == 0 } {
    set PSC_BASE 0x1C10000
  } else {
    set PSC_BASE 0x1E27000
  }

  set PSC_MDSTAT [ expr $PSC_BASE + 0x0800 ]
  set PSC_MDCTL  [ expr $PSC_BASE + 0x0A00 ]
  set PSC_PTCMD  [ expr $PSC_BASE + 0x0120 ]
  set PSC_PTSTAT [ expr $PSC_BASE + 0x0128 ]

  # PSCx_MDSTAT
  set rval [ ReadLong $DEV [ expr $PSC_MDSTAT+(4*$LPSC_num) ] 0 ]
  if { ($rval & $MS) != $DT } {

    # MDCTL
    set rval [ ReadLong $DEV [ expr $PSC_MDCTL+(4*$LPSC_num) ] 0 ]
    set rval [ expr ($rval & (~$MS)) | $DT ]
    PutLong $DEV [ expr $PSC_MDCTL+(4*$LPSC_num) ] 0 $rval

    # PTCTL
    PutLong $DEV $PSC_PTCMD 0 [ expr 1 << $PD ]

    # Wait for power state transition to finish
    set jj 0
    while { $jj < 200 } {
      set rval [ ReadLong $DEV $PSC_PTSTAT 0 ]
      if { ($rval & (1 << $PD)) == 0 } {
        break
      }
      incr jj
    } 
    if { $jj >= 200 } {
      xerror $DEV "Timeout waitig power state transition" 1
    }

    # Verify new state
    set jj 0
    while { $jj < 200 } {
      set rval [ ReadLong $DEV [ expr $PSC_MDSTAT+(4*$LPSC_num) ] 0 ]
      if { ($rval & $MS) == $DT } {
        break
      }
      incr jj
    } 
    if { $jj >= 200 } {
      xerror $DEV "Timeout verifying new state" 1
    }

    puts "PSC($PSC_NUM) LPSC($LPSC_num) enabled"
    
  } else {

    puts "PSC($PSC_NUM) LPSC($LPSC_num) already enabled"

  }

}

# L-138: ARM core starts first, we need start DSP core

proc UserCallBeforeConnect { dev } {

  upvar 1 ARM ARM

  # override 'root' endianess
  set TargetBigEndian 3

  set result  [xds connect $ARM]

  if { $result == 0 } {
    puts "Connected to ARM"
  } else {
    xerror $ARM "Error connecting to ARM" 1
  }

  # reset device
  xds reset $ARM
  puts "ARM Core reset complete"

  # unlock KICK
  set KICK0R [expr 0x01C14000 + 0x38]
  set KICK1R [expr 0x01C14000 + 0x3C]
  PutLong $ARM $KICK0R 0 0x83e70b13
  PutLong $ARM $KICK1R 0 0x95a4f1e0
  puts "KICK unlocked"

  # Enable needed PSCs

  # EMIFA
  PSC_LPSC_Enable $ARM 0 0 3 0
  # SPI0
  PSC_LPSC_Enable $ARM 0 0 4 0
  # AINTC
  PSC_LPSC_Enable $ARM 0 0 6 0
  # RAMROM
  PSC_LPSC_Enable $ARM 0 0 7 0

  # GPIO
  PSC_LPSC_Enable $ARM 1 0 3 0
  # SPI1
  PSC_LPSC_Enable $ARM 1 0 10 0
  # I2C1
  PSC_LPSC_Enable $ARM 1 0 11 0
  # BR
  PSC_LPSC_Enable $ARM 1 0 30 0
  # SHARED_RAM
  PSC_LPSC_Enable $ARM 1 0 31 0

  # put PLL0 into BYPASS mode

  set CFGCHIP0 [expr 0x01C14000 + 0x17C]
  set PLL0_PLLCTL [expr 0x01C11000 + 0x100]

  set rval [ ReadLong $ARM $CFGCHIP0 0 ]
  set rval [ expr $rval & ~0x00000010 ]
  PutLong $ARM $CFGCHIP0 0 $rval

  set rval [ ReadLong $ARM $PLL0_PLLCTL 0 ]
  set rval [ expr $rval & ~0x00000020 ]
  PutLong $ARM $PLL0_PLLCTL 0 $rval

  set rval [ ReadLong $ARM $PLL0_PLLCTL 0 ]
  set rval [ expr $rval & ~0x00000200 ]
  PutLong $ARM $PLL0_PLLCTL 0 $rval

  set rval [ ReadLong $ARM $PLL0_PLLCTL 0 ]
  set rval [ expr $rval & ~0x00000001 ]
  PutLong $ARM $PLL0_PLLCTL 0 $rval

  puts "PLL0 is set to BYPASS mode"
  
  # Put "BNOP $" - opcode 000ca120
  # into Shared RAM
  PutLong $ARM 0x80000000 0 0x000ca120
  set rval [ ReadLong $ARM 0x80000000 0 ]
  if { $rval != 0x000ca120 } {
    puts [format "Error storing DSP opcode (0x%08X)" $rval]
  } else {
    puts "DSP infinite loop opcode stored"
  }

  # HOST1CFG
  PutLong $ARM 0x01C14044 0 0x80000000

  # DSP
  PSC_LPSC_Enable $ARM 0 1 15 1

  puts "DSP wakeup and prepare complete"

  xds disconnect $ARM
  xds quit $ARM

}


proc CPUSpecificInit { dev } {

}
